@@ -28,14 +28,16 @@
 
 #include "DataSparse.h"
 
-DataSparse::DataSparse() :
-    data(0) {
+namespace ranger {
+
+DataSparse::DataSparse(Eigen::SparseMatrix<double>& data, std::vector<std::string> variable_names, size_t num_rows,
+    size_t num_cols) :
+    data { } {
+  this->data.swap(data);
+  this->variable_names = variable_names;
+  this->num_rows = num_rows;
+  this->num_cols = num_cols;
+  this->num_cols_no_snp = num_cols;
 }
 
-DataSparse::~DataSparse() {
-  if (!externalData) {
-    delete[] data;
-  }
-}
-
-
+} // namespace ranger
