@@ -1,29 +1,12 @@
 /*-------------------------------------------------------------------------------
- This file is part of Ranger.
+ This file is part of ranger.
 
- Ranger is free software: you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation, either version 3 of the License, or
- (at your option) any later version.
+ Copyright (c) [2014-2018] [Marvin N. Wright]
 
- Ranger is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- GNU General Public License for more details.
+ This software may be modified and distributed under the terms of the MIT license.
 
- You should have received a copy of the GNU General Public License
- along with Ranger. If not, see <http://www.gnu.org/licenses/>.
-
- Written by:
-
- Marvin N. Wright
- Institut für Medizinische Biometrie und Statistik
- Universität zu Lübeck
- Ratzeburger Allee 160
- 23562 Lübeck
- Germany
-
- http://www.imbs-luebeck.de
+ Please note that the C++ core of ranger is distributed under MIT license and the
+ R package "ranger" under GPL3 license.
  #-------------------------------------------------------------------------------*/
 
 #include <unordered_map>
@@ -39,11 +22,7 @@
 #include "TreeClassification.h"
 #include "Data.h"
 
-ForestClassification::ForestClassification() {
-}
-
-ForestClassification::~ForestClassification() {
-}
+namespace ranger {
 
 void ForestClassification::loadForest(size_t dependent_varID, size_t num_trees,
     std::vector<std::vector<std::vector<size_t>> >& forest_child_nodeIDs,
@@ -58,9 +37,9 @@ void ForestClassification::loadForest(size_t dependent_varID, size_t num_trees,
   // Create trees
   trees.reserve(num_trees);
   for (size_t i = 0; i < num_trees; ++i) {
-    Tree* tree = new TreeClassification(forest_child_nodeIDs[i], forest_split_varIDs[i], forest_split_values[i],
-        &this->class_values, &response_classIDs);
-    trees.push_back(tree);
+    trees.push_back(
+        make_unique<TreeClassification>(forest_child_nodeIDs[i], forest_split_varIDs[i], forest_split_values[i],
+            &this->class_values, &response_classIDs));
   }
 
   // Create thread ranges
@@ -94,6 +73,21 @@ void ForestClassification::initInternal(std::string status_variable_name) {
     }
   }
 
+  // Create sampleIDs_per_class if required
+  if (sample_fraction.size() > 1) {
+    sampleIDs_per_class.resize(sample_fraction.size());
+    for (auto& v : sampleIDs_per_class) {
+      v.reserve(num_samples);
+    }
+    for (size_t i = 0; i < num_samples; ++i) {
+      size_t classID = response_classIDs[i];
+      sampleIDs_per_class[classID].push_back(i);
+    }
+  }
+
+  // Set class weights all to 1
+  class_weights = std::vector<double>(class_values.size(), 1.0);
+
   // Sort data if memory saving mode
   if (!memory_saving_splitting) {
     data->sort();
@@ -103,7 +97,8 @@ void ForestClassification::initInternal(std::string status_variable_name) {
 void ForestClassification::growInternal() {
   trees.reserve(num_trees);
   for (size_t i = 0; i < num_trees; ++i) {
-    trees.push_back(new TreeClassification(&class_values, &response_classIDs));
+    trees.push_back(
+        make_unique<TreeClassification>(&class_values, &response_classIDs, &sampleIDs_per_class, &class_weights));
   }
 }
 
@@ -123,18 +118,16 @@ void ForestClassification::predictInternal(size_t sample_idx) {
     // Get all tree predictions
     for (size_t tree_idx = 0; tree_idx < num_trees; ++tree_idx) {
       if (prediction_type == TERMINALNODES) {
-        predictions[0][sample_idx][tree_idx] = ((TreeClassification*) trees[tree_idx])->getPredictionTerminalNodeID(
-            sample_idx);
+        predictions[0][sample_idx][tree_idx] = getTreePredictionTerminalNodeID(tree_idx, sample_idx);
       } else {
-        predictions[0][sample_idx][tree_idx] = ((TreeClassification*) trees[tree_idx])->getPrediction(sample_idx);
+        predictions[0][sample_idx][tree_idx] = getTreePrediction(tree_idx, sample_idx);
       }
     }
   } else {
     // Count classes over trees and save class with maximum count
     std::unordered_map<double, size_t> class_count;
     for (size_t tree_idx = 0; tree_idx < num_trees; ++tree_idx) {
-      double value = ((TreeClassification*) trees[tree_idx])->getPrediction(sample_idx);
-      ++class_count[value];
+      ++class_count[getTreePrediction(tree_idx, sample_idx)];
     }
     predictions[0][0][sample_idx] = mostFrequentValue(class_count, random_number_generator);
   }
@@ -153,8 +146,7 @@ void ForestClassification::computePredictionErrorInternal() {
   for (size_t tree_idx = 0; tree_idx < num_trees; ++tree_idx) {
     for (size_t sample_idx = 0; sample_idx < trees[tree_idx]->getNumSamplesOob(); ++sample_idx) {
       size_t sampleID = trees[tree_idx]->getOobSampleIDs()[sample_idx];
-      double value = ((TreeClassification*) trees[tree_idx])->getPrediction(sample_idx);
-      ++class_counts[sampleID][value];
+      ++class_counts[sampleID][getTreePrediction(tree_idx, sample_idx)];
     }
   }
 
@@ -188,7 +180,9 @@ void ForestClassification::computePredictionErrorInternal() {
 
 // #nocov start
 void ForestClassification::writeOutputInternal() {
-  *verbose_out << "Tree type:                         " << "Classification" << std::endl;
+  if (verbose_out) {
+    *verbose_out << "Tree type:                         " << "Classification" << std::endl;
+  }
 }
 
 void ForestClassification::writeConfusionFile() {
@@ -231,7 +225,8 @@ void ForestClassification::writeConfusionFile() {
   }
 
   outfile.close();
-  *verbose_out << "Saved confusion matrix to file " << filename << "." << std::endl;
+  if (verbose_out)
+    *verbose_out << "Saved confusion matrix to file " << filename << "." << std::endl;
 }
 
 void ForestClassification::writePredictionFile() {
@@ -266,7 +261,8 @@ void ForestClassification::writePredictionFile() {
     }
   }
 
-  *verbose_out << "Saved predictions to file " << filename << "." << std::endl;
+  if (verbose_out)
+    *verbose_out << "Saved predictions to file " << filename << "." << std::endl;
 }
 
 void ForestClassification::saveToFileInternal(std::ofstream& outfile) {
@@ -318,8 +314,21 @@ void ForestClassification::loadFromFileInternal(std::ifstream& infile) {
     }
 
     // Create tree
-    Tree* tree = new TreeClassification(child_nodeIDs, split_varIDs, split_values, &class_values, &response_classIDs);
-    trees.push_back(tree);
+    trees.push_back(
+        make_unique<TreeClassification>(child_nodeIDs, split_varIDs, split_values, &class_values, &response_classIDs));
   }
 }
+
+double ForestClassification::getTreePrediction(size_t tree_idx, size_t sample_idx) const {
+  const auto& tree = dynamic_cast<const TreeClassification&>(*trees[tree_idx]);
+  return tree.getPrediction(sample_idx);
+}
+
+size_t ForestClassification::getTreePredictionTerminalNodeID(size_t tree_idx, size_t sample_idx) const {
+  const auto& tree = dynamic_cast<const TreeClassification&>(*trees[tree_idx]);
+  return tree.getPredictionTerminalNodeID(sample_idx);
+}
+
 // #nocov end
+
+}// namespace ranger
