@@ -18,27 +18,29 @@ namespace ranger {
 
 Tree::Tree() :
     dependent_varID(0), mtry(0), num_samples(0), num_samples_oob(0), min_node_size(0), deterministic_varIDs(0), split_select_varIDs(
-        0), split_select_weights(0), case_weights(0), oob_sampleIDs(0), holdout(false), keep_inbag(false), data(0), variable_importance(
-        0), importance_mode(DEFAULT_IMPORTANCE_MODE), sample_with_replacement(true), sample_fraction(0), memory_saving_splitting(
-        false), splitrule(DEFAULT_SPLITRULE), alpha(DEFAULT_ALPHA), minprop(DEFAULT_MINPROP), num_random_splits(
-        DEFAULT_NUM_RANDOM_SPLITS) {
+        0), split_select_weights(0), case_weights(0), manual_inbag(0), oob_sampleIDs(0), holdout(false), keep_inbag(
+        false), data(0), variable_importance(0), importance_mode(DEFAULT_IMPORTANCE_MODE), sample_with_replacement(
+        true), sample_fraction(0), memory_saving_splitting(false), splitrule(DEFAULT_SPLITRULE), alpha(DEFAULT_ALPHA), minprop(
+        DEFAULT_MINPROP), num_random_splits(DEFAULT_NUM_RANDOM_SPLITS), max_depth(DEFAULT_MAXDEPTH), depth(0), last_left_nodeID(
+        0) {
 }
 
 Tree::Tree(std::vector<std::vector<size_t>>& child_nodeIDs, std::vector<size_t>& split_varIDs,
     std::vector<double>& split_values) :
     dependent_varID(0), mtry(0), num_samples(0), num_samples_oob(0), min_node_size(0), deterministic_varIDs(0), split_select_varIDs(
-        0), split_select_weights(0), case_weights(0), split_varIDs(split_varIDs), split_values(split_values), child_nodeIDs(
-        child_nodeIDs), oob_sampleIDs(0), holdout(false), keep_inbag(false), data(0), variable_importance(0), importance_mode(
-        DEFAULT_IMPORTANCE_MODE), sample_with_replacement(true), sample_fraction(0), memory_saving_splitting(false), splitrule(
-        DEFAULT_SPLITRULE), alpha(DEFAULT_ALPHA), minprop(DEFAULT_MINPROP), num_random_splits(DEFAULT_NUM_RANDOM_SPLITS) {
+        0), split_select_weights(0), case_weights(0), manual_inbag(0), split_varIDs(split_varIDs), split_values(
+        split_values), child_nodeIDs(child_nodeIDs), oob_sampleIDs(0), holdout(false), keep_inbag(false), data(0), variable_importance(
+        0), importance_mode(DEFAULT_IMPORTANCE_MODE), sample_with_replacement(true), sample_fraction(0), memory_saving_splitting(
+        false), splitrule(DEFAULT_SPLITRULE), alpha(DEFAULT_ALPHA), minprop(DEFAULT_MINPROP), num_random_splits(
+        DEFAULT_NUM_RANDOM_SPLITS), max_depth(DEFAULT_MAXDEPTH), depth(0), last_left_nodeID(0) {
 }
 
 void Tree::init(const Data* data, uint mtry, size_t dependent_varID, size_t num_samples, uint seed,
     std::vector<size_t>* deterministic_varIDs, std::vector<size_t>* split_select_varIDs,
     std::vector<double>* split_select_weights, ImportanceMode importance_mode, uint min_node_size,
     bool sample_with_replacement, bool memory_saving_splitting, SplitRule splitrule, std::vector<double>* case_weights,
-    bool keep_inbag, std::vector<double>* sample_fraction, double alpha, double minprop, bool holdout,
-    uint num_random_splits) {
+    std::vector<size_t>* manual_inbag, bool keep_inbag, std::vector<double>* sample_fraction, double alpha,
+    double minprop, bool holdout, uint num_random_splits, uint max_depth) {
 
   this->data = data;
   this->mtry = mtry;
@@ -62,12 +64,14 @@ void Tree::init(const Data* data, uint mtry, size_t dependent_varID, size_t num_
   this->sample_with_replacement = sample_with_replacement;
   this->splitrule = splitrule;
   this->case_weights = case_weights;
+  this->manual_inbag = manual_inbag;
   this->keep_inbag = keep_inbag;
   this->sample_fraction = sample_fraction;
   this->holdout = holdout;
   this->alpha = alpha;
   this->minprop = minprop;
   this->num_random_splits = num_random_splits;
+  this->max_depth = max_depth;
 }
 
 void Tree::grow(std::vector<double>* variable_importance) {
@@ -89,6 +93,8 @@ void Tree::grow(std::vector<double>* variable_importance) {
     } else {
       bootstrapWithoutReplacementClassWise();
     }
+  } else if (!manual_inbag->empty()) {
+    setManualInbag();
   } else {
     if (sample_with_replacement) {
       bootstrap();
@@ -97,15 +103,27 @@ void Tree::grow(std::vector<double>* variable_importance) {
     }
   }
 
-// While not all nodes terminal, split next node
+  // Init start and end positions
+  start_pos[0] = 0;
+  end_pos[0] = sampleIDs.size();
+
+  // While not all nodes terminal, split next node
   size_t num_open_nodes = 1;
   size_t i = 0;
+  depth = 0;
   while (num_open_nodes > 0) {
+    // Split node
     bool is_terminal_node = splitNode(i);
     if (is_terminal_node) {
       --num_open_nodes;
     } else {
       ++num_open_nodes;
+      if (i >= last_left_nodeID) {
+        // If new level, increase depth
+        // (left_node saves left-most node in current level, new level reached if that node is splitted)
+        last_left_nodeID = split_varIDs.size() - 2;
+        ++depth;
+      }
     }
     ++i;
   }
@@ -233,18 +251,24 @@ void Tree::createPossibleSplitVarSubset(std::vector<size_t>& result) {
     num_vars += data->getNumCols() - data->getNoSplitVariables().size();
   }
 
-  // Always use deterministic variables
-  std::copy(deterministic_varIDs->begin(), deterministic_varIDs->end(), std::inserter(result, result.end()));
-
   // Randomly add non-deterministic variables (according to weights if needed)
   if (split_select_weights->empty()) {
-    drawWithoutReplacementSkip(result, random_number_generator, num_vars, data->getNoSplitVariables(), mtry);
+    if (deterministic_varIDs->empty()) {
+      drawWithoutReplacementSkip(result, random_number_generator, num_vars, data->getNoSplitVariables(), mtry);
+    } else {
+      std::vector<size_t> skip;
+      std::copy(data->getNoSplitVariables().begin(), data->getNoSplitVariables().end(),
+          std::inserter(skip, skip.end()));
+      std::copy(deterministic_varIDs->begin(), deterministic_varIDs->end(), std::inserter(skip, skip.end()));
+      std::sort(skip.begin(), skip.end());
+      drawWithoutReplacementSkip(result, random_number_generator, num_vars, skip, mtry);
+    }
   } else {
-    // No corrected Gini importance supported for weighted splitting
-    size_t num_draws = mtry - result.size();
-    drawWithoutReplacementWeighted(result, random_number_generator, *split_select_varIDs, num_draws,
-        *split_select_weights);
+    drawWithoutReplacementWeighted(result, random_number_generator, *split_select_varIDs, mtry, *split_select_weights);
   }
+
+  // Always use deterministic variables
+  std::copy(deterministic_varIDs->begin(), deterministic_varIDs->end(), std::inserter(result, result.end()));
 }
 
 bool Tree::splitNode(size_t nodeID) {
@@ -267,41 +291,56 @@ bool Tree::splitNode(size_t nodeID) {
   split_varIDs[nodeID] = data->getUnpermutedVarID(split_varID);
 
   // Create child nodes
-  size_t left_child_nodeID = sampleIDs.size();
+  size_t left_child_nodeID = split_varIDs.size();
   child_nodeIDs[0][nodeID] = left_child_nodeID;
   createEmptyNode();
+  start_pos[left_child_nodeID] = start_pos[nodeID];
 
-  size_t right_child_nodeID = sampleIDs.size();
+  size_t right_child_nodeID = split_varIDs.size();
   child_nodeIDs[1][nodeID] = right_child_nodeID;
   createEmptyNode();
+  start_pos[right_child_nodeID] = end_pos[nodeID];
 
   // For each sample in node, assign to left or right child
   if (data->isOrderedVariable(split_varID)) {
     // Ordered: left is <= splitval and right is > splitval
-    for (auto& sampleID : sampleIDs[nodeID]) {
+    size_t pos = start_pos[nodeID];
+    while (pos < start_pos[right_child_nodeID]) {
+      size_t sampleID = sampleIDs[pos];
       if (data->get(sampleID, split_varID) <= split_value) {
-        sampleIDs[left_child_nodeID].push_back(sampleID);
+        // If going to left, do nothing
+        ++pos;
       } else {
-        sampleIDs[right_child_nodeID].push_back(sampleID);
+        // If going to right, move to right end
+        --start_pos[right_child_nodeID];
+        std::swap(sampleIDs[pos], sampleIDs[start_pos[right_child_nodeID]]);
       }
     }
   } else {
     // Unordered: If bit at position is 1 -> right, 0 -> left
-    for (auto& sampleID : sampleIDs[nodeID]) {
-
+    size_t pos = start_pos[nodeID];
+    while (pos < start_pos[right_child_nodeID]) {
+      size_t sampleID = sampleIDs[pos];
       double level = data->get(sampleID, split_varID);
       size_t factorID = floor(level) - 1;
       size_t splitID = floor(split_value);
 
       // Left if 0 found at position factorID
       if (!(splitID & (1 << factorID))) {
-        sampleIDs[left_child_nodeID].push_back(sampleID);
+        // If going to left, do nothing
+        ++pos;
       } else {
-        sampleIDs[right_child_nodeID].push_back(sampleID);
+        // If going to right, move to right end
+        --start_pos[right_child_nodeID];
+        std::swap(sampleIDs[pos], sampleIDs[start_pos[right_child_nodeID]]);
       }
     }
   }
 
+  // End position of left child is start position of right child
+  end_pos[left_child_nodeID] = start_pos[right_child_nodeID];
+  end_pos[right_child_nodeID] = end_pos[nodeID];
+
   // No terminal node
   return false;
 }
@@ -311,7 +350,8 @@ void Tree::createEmptyNode() {
   split_values.push_back(0);
   child_nodeIDs[0].push_back(0);
   child_nodeIDs[1].push_back(0);
-  sampleIDs.push_back(std::vector<size_t>());
+  start_pos.push_back(0);
+  end_pos.push_back(0);
 
   createEmptyNodeInternal();
 }
@@ -376,7 +416,7 @@ void Tree::bootstrap() {
   size_t num_samples_inbag = (size_t) num_samples * (*sample_fraction)[0];
 
 // Reserve space, reserve a little more to be save)
-  sampleIDs[0].reserve(num_samples_inbag);
+  sampleIDs.reserve(num_samples_inbag);
   oob_sampleIDs.reserve(num_samples * (exp(-(*sample_fraction)[0]) + 0.1));
 
   std::uniform_int_distribution<size_t> unif_dist(0, num_samples - 1);
@@ -387,7 +427,7 @@ void Tree::bootstrap() {
 // Draw num_samples samples with replacement (num_samples_inbag out of n) as inbag and mark as not OOB
   for (size_t s = 0; s < num_samples_inbag; ++s) {
     size_t draw = unif_dist(random_number_generator);
-    sampleIDs[0].push_back(draw);
+    sampleIDs.push_back(draw);
     ++inbag_counts[draw];
   }
 
@@ -411,7 +451,7 @@ void Tree::bootstrapWeighted() {
   size_t num_samples_inbag = (size_t) num_samples * (*sample_fraction)[0];
 
 // Reserve space, reserve a little more to be save)
-  sampleIDs[0].reserve(num_samples_inbag);
+  sampleIDs.reserve(num_samples_inbag);
   oob_sampleIDs.reserve(num_samples * (exp(-(*sample_fraction)[0]) + 0.1));
 
   std::discrete_distribution<> weighted_dist(case_weights->begin(), case_weights->end());
@@ -422,7 +462,7 @@ void Tree::bootstrapWeighted() {
 // Draw num_samples samples with replacement (n out of n) as inbag and mark as not OOB
   for (size_t s = 0; s < num_samples_inbag; ++s) {
     size_t draw = weighted_dist(random_number_generator);
-    sampleIDs[0].push_back(draw);
+    sampleIDs.push_back(draw);
     ++inbag_counts[draw];
   }
 
@@ -452,7 +492,7 @@ void Tree::bootstrapWithoutReplacement() {
 
 // Use fraction (default 63.21%) of the samples
   size_t num_samples_inbag = (size_t) num_samples * (*sample_fraction)[0];
-  shuffleAndSplit(sampleIDs[0], oob_sampleIDs, num_samples, num_samples_inbag, random_number_generator);
+  shuffleAndSplit(sampleIDs, oob_sampleIDs, num_samples, num_samples_inbag, random_number_generator);
   num_samples_oob = oob_sampleIDs.size();
 
   if (keep_inbag) {
@@ -468,12 +508,11 @@ void Tree::bootstrapWithoutReplacementWeighted() {
 
 // Use fraction (default 63.21%) of the samples
   size_t num_samples_inbag = (size_t) num_samples * (*sample_fraction)[0];
-  drawWithoutReplacementWeighted(sampleIDs[0], random_number_generator, num_samples - 1, num_samples_inbag,
-      *case_weights);
+  drawWithoutReplacementWeighted(sampleIDs, random_number_generator, num_samples - 1, num_samples_inbag, *case_weights);
 
 // All observation are 0 or 1 times inbag
   inbag_counts.resize(num_samples, 0);
-  for (auto& sampleID : sampleIDs[0]) {
+  for (auto& sampleID : sampleIDs) {
     inbag_counts[sampleID] = 1;
   }
 
@@ -507,4 +546,30 @@ void Tree::bootstrapWithoutReplacementClassWise() {
   // Empty on purpose (virtual function only implemented in classification and probability)
 }
 
+void Tree::setManualInbag() {
+  // Select observation as specified in manual_inbag vector
+  sampleIDs.reserve(manual_inbag->size());
+  inbag_counts.resize(num_samples, 0);
+  for (size_t i = 0; i < manual_inbag->size(); ++i) {
+    size_t inbag_count = (*manual_inbag)[i];
+    if ((*manual_inbag)[i] > 0) {
+      for (size_t j = 0; j < inbag_count; ++j) {
+        sampleIDs.push_back(i);
+      }
+      inbag_counts[i] = inbag_count;
+    } else {
+      oob_sampleIDs.push_back(i);
+    }
+  }
+  num_samples_oob = oob_sampleIDs.size();
+
+  // Shuffle samples
+  std::shuffle(sampleIDs.begin(), sampleIDs.end(), random_number_generator);
+
+  if (!keep_inbag) {
+    inbag_counts.clear();
+    inbag_counts.shrink_to_fit();
+  }
+}
+
 } // namespace ranger
