@@ -45,9 +45,10 @@ double TreeRegression::estimate(size_t nodeID) {
 
 // Mean of responses of samples in node
   double sum_responses_in_node = 0;
-  size_t num_samples_in_node = sampleIDs[nodeID].size();
-  for (size_t i = 0; i < sampleIDs[nodeID].size(); ++i) {
-    sum_responses_in_node += data->get(sampleIDs[nodeID][i], dependent_varID);
+  size_t num_samples_in_node = end_pos[nodeID] - start_pos[nodeID];
+  for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
+    sum_responses_in_node += data->get(sampleID, dependent_varID);
   }
   return (sum_responses_in_node / (double) num_samples_in_node);
 }
@@ -58,8 +59,10 @@ void TreeRegression::appendToFileInternal(std::ofstream& file) { // #nocov start
 
 bool TreeRegression::splitNodeInternal(size_t nodeID, std::vector<size_t>& possible_split_varIDs) {
 
-  // Check node size, stop if maximum reached
-  if (sampleIDs[nodeID].size() <= min_node_size) {
+  size_t num_samples_node = end_pos[nodeID] - start_pos[nodeID];
+
+  // Stop if maximum node size or depth reached
+  if (num_samples_node <= min_node_size || (nodeID >= last_left_nodeID && max_depth > 0 && depth >= max_depth)) {
     split_values[nodeID] = estimate(nodeID);
     return true;
   }
@@ -67,9 +70,10 @@ bool TreeRegression::splitNodeInternal(size_t nodeID, std::vector<size_t>& possi
   // Check if node is pure and set split_value to estimate and stop if pure
   bool pure = true;
   double pure_value = 0;
-  for (size_t i = 0; i < sampleIDs[nodeID].size(); ++i) {
-    double value = data->get(sampleIDs[nodeID][i], dependent_varID);
-    if (i != 0 && value != pure_value) {
+  for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
+    double value = data->get(sampleID, dependent_varID);
+    if (pos != start_pos[nodeID] && value != pure_value) {
       pure = false;
       break;
     }
@@ -119,14 +123,15 @@ double TreeRegression::computePredictionAccuracyInternal() {
 
 bool TreeRegression::findBestSplit(size_t nodeID, std::vector<size_t>& possible_split_varIDs) {
 
-  size_t num_samples_node = sampleIDs[nodeID].size();
+  size_t num_samples_node = end_pos[nodeID] - start_pos[nodeID];
   double best_decrease = -1;
   size_t best_varID = 0;
   double best_value = 0;
 
   // Compute sum of responses in node
   double sum_node = 0;
-  for (auto& sampleID : sampleIDs[nodeID]) {
+  for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
     sum_node += data->get(sampleID, dependent_varID);
   }
 
@@ -174,7 +179,7 @@ void TreeRegression::findBestSplitValueSmallQ(size_t nodeID, size_t varID, doubl
 
   // Create possible split values
   std::vector<double> possible_split_values;
-  data->getAllValues(possible_split_values, sampleIDs[nodeID], varID);
+  data->getAllValues(possible_split_values, sampleIDs, varID, start_pos[nodeID], end_pos[nodeID]);
 
   // Try next variable if all equal for this
   if (possible_split_values.size() < 2) {
@@ -203,7 +208,8 @@ void TreeRegression::findBestSplitValueSmallQ(size_t nodeID, size_t varID, doubl
   const size_t num_splits = possible_split_values.size() - 1;
 
   // Sum in right child and possbile split
-  for (auto& sampleID : sampleIDs[nodeID]) {
+  for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
     double value = data->get(sampleID, varID);
     double response = data->get(sampleID, dependent_varID);
 
@@ -253,7 +259,8 @@ void TreeRegression::findBestSplitValueLargeQ(size_t nodeID, size_t varID, doubl
   std::fill_n(counter.begin(), num_unique, 0);
   std::fill_n(sums.begin(), num_unique, 0);
 
-  for (auto& sampleID : sampleIDs[nodeID]) {
+  for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
     size_t index = data->getIndex(sampleID, varID);
 
     sums[index] += data->get(sampleID, dependent_varID);
@@ -309,7 +316,7 @@ void TreeRegression::findBestSplitValueUnordered(size_t nodeID, size_t varID, do
 
 // Create possible split values
   std::vector<double> factor_levels;
-  data->getAllValues(factor_levels, sampleIDs[nodeID], varID);
+  data->getAllValues(factor_levels, sampleIDs, varID, start_pos[nodeID], end_pos[nodeID]);
 
 // Try next variable if all equal for this
   if (factor_levels.size() < 2) {
@@ -339,7 +346,8 @@ void TreeRegression::findBestSplitValueUnordered(size_t nodeID, size_t varID, do
     size_t n_right = 0;
 
     // Sum in right child
-    for (auto& sampleID : sampleIDs[nodeID]) {
+    for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+      size_t sampleID = sampleIDs[pos];
       double response = data->get(sampleID, dependent_varID);
       double value = data->get(sampleID, varID);
       size_t factorID = floor(value) - 1;
@@ -368,12 +376,13 @@ void TreeRegression::findBestSplitValueUnordered(size_t nodeID, size_t varID, do
 
 bool TreeRegression::findBestSplitMaxstat(size_t nodeID, std::vector<size_t>& possible_split_varIDs) {
 
-  size_t num_samples_node = sampleIDs[nodeID].size();
+  size_t num_samples_node = end_pos[nodeID] - start_pos[nodeID];
 
   // Compute ranks
   std::vector<double> response;
   response.reserve(num_samples_node);
-  for (auto& sampleID : sampleIDs[nodeID]) {
+  for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
     response.push_back(data->get(sampleID, dependent_varID));
   }
   std::vector<double> ranks = rank(response);
@@ -394,7 +403,8 @@ bool TreeRegression::findBestSplitMaxstat(size_t nodeID, std::vector<size_t>& po
     // Get all observations
     std::vector<double> x;
     x.reserve(num_samples_node);
-    for (auto& sampleID : sampleIDs[nodeID]) {
+    for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+      size_t sampleID = sampleIDs[pos];
       x.push_back(data->get(sampleID, varID));
     }
 
@@ -469,14 +479,15 @@ bool TreeRegression::findBestSplitMaxstat(size_t nodeID, std::vector<size_t>& po
 
 bool TreeRegression::findBestSplitExtraTrees(size_t nodeID, std::vector<size_t>& possible_split_varIDs) {
 
-  size_t num_samples_node = sampleIDs[nodeID].size();
+  size_t num_samples_node = end_pos[nodeID] - start_pos[nodeID];
   double best_decrease = -1;
   size_t best_varID = 0;
   double best_value = 0;
 
   // Compute sum of responses in node
   double sum_node = 0;
-  for (auto& sampleID : sampleIDs[nodeID]) {
+  for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
     sum_node += data->get(sampleID, dependent_varID);
   }
 
@@ -514,7 +525,7 @@ void TreeRegression::findBestSplitValueExtraTrees(size_t nodeID, size_t varID, d
   // Get min/max values of covariate in node
   double min;
   double max;
-  data->getMinMaxValues(min, max, sampleIDs[nodeID], varID);
+  data->getMinMaxValues(min, max, sampleIDs, varID, start_pos[nodeID], end_pos[nodeID]);
 
   // Try next variable if all equal for this
   if (min == max) {
@@ -549,7 +560,8 @@ void TreeRegression::findBestSplitValueExtraTrees(size_t nodeID, size_t varID, d
   const size_t num_splits = possible_split_values.size();
 
   // Sum in right child and possbile split
-  for (auto& sampleID : sampleIDs[nodeID]) {
+  for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
     double value = data->get(sampleID, varID);
     double response = data->get(sampleID, dependent_varID);
 
@@ -593,7 +605,8 @@ void TreeRegression::findBestSplitValueExtraTreesUnordered(size_t nodeID, size_t
 
   // Get all factor indices in node
   std::vector<bool> factor_in_node(num_unique_values, false);
-  for (auto& sampleID : sampleIDs[nodeID]) {
+  for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
     size_t index = data->getIndex(sampleID, varID);
     factor_in_node[index] = true;
   }
@@ -649,7 +662,8 @@ void TreeRegression::findBestSplitValueExtraTreesUnordered(size_t nodeID, size_t
     size_t n_right = 0;
 
     // Sum in right child
-    for (auto& sampleID : sampleIDs[nodeID]) {
+    for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+      size_t sampleID = sampleIDs[pos];
       double response = data->get(sampleID, dependent_varID);
       double value = data->get(sampleID, varID);
       size_t factorID = floor(value) - 1;
@@ -678,13 +692,16 @@ void TreeRegression::findBestSplitValueExtraTreesUnordered(size_t nodeID, size_t
 
 void TreeRegression::addImpurityImportance(size_t nodeID, size_t varID, double decrease) {
 
+  size_t num_samples_node = end_pos[nodeID] - start_pos[nodeID];
+
   double best_decrease = decrease;
   if (splitrule != MAXSTAT) {
     double sum_node = 0;
-    for (auto& sampleID : sampleIDs[nodeID]) {
+    for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+      size_t sampleID = sampleIDs[pos];
       sum_node += data->get(sampleID, dependent_varID);
     }
-    best_decrease = decrease - sum_node * sum_node / (double) sampleIDs[nodeID].size();
+    best_decrease = decrease - sum_node * sum_node / (double) num_samples_node;
   }
 
   // No variable importance for no split variables
